import processing.opengl.*;
ArcBall arcball;
int [] boxFace = {
1,3,5,7,
5,6,8,7,
6,4,3,5,
4,2,1,3,
1,7,8,2,
4,6,8,2
};
float wide = 640.0;
float high = 480.0;
float deep = 480.0;
float xc,yc,zc;
VectorPath [] path;
Buttons [] button;
Cylinder [][] cylinders;
boolean record;
int zoom;
float rot;
int [] cols = {
#CCCC55, #FF0000, #00FF00, #00FFFF, #FFFF00, #0000FF
};
String [] label = {
  "mustard?0","red1","green2","cyan3","yellow4","blue5"
};
boolean sim = false;
boolean showShell = false;
PFont font;
// oscP5 instance for the osc communication
OscP5 oscP5;
int receiveAtPort;
int sendToPort;
String host;
String oscP5event;

void setup(){
  size(800, 600, P3D);
  xc = wide/2;
  yc = high/2;
  zc = deep/2;
  background(0);
  stroke(255);
  path = new VectorPath[6];
  button = new Buttons[12];
  cylinders = new Cylinder[6][];
  font = loadFont("ArialMT-48.vlw");
  textFont(font,12);
  arcball = new ArcBall(wide / 2.0f, high / 2.0f, min(wide - 20, high - 20) / 2.0f);
  for(int i = 0; i < path.length; i++){
    path[i] = new VectorPath(cols[i]);
    button[i] = new Buttons(width-200,100+20*i,200,20,cols[i],label[i]+": false",null); 
  }
button[6] = new Buttons(width-200,220,200,20,#EEEEEE,"record: false",null);
button[7] = new Buttons(width-200,240,200,20,#EEEEEE,"reset",null);
button[8] = new Buttons(width-200,260,200,20,#EEEEEE,"simulate: false",null);
button[9] = new Buttons(width-200,280,200,20,#EEEEEE,"create shell",null);
button[10] = new Buttons(width-200,300,200,20,#EEEEEE,"show shell: false",null);
button[11] = new Buttons(width-200,320,200,20,#EEEEEE,"save shell",null);
  initOsc();
  ellipseMode(CENTER);
}

void draw(){
  background(0);
  directionalLight(200,200,200,0,0,-1);
  pointLight(255,255,255,width<<1,height<<1,0);
  pushMatrix();
  translate(xc, yc, -400.0f+zoom);
  arcball.run();
  for(int i = 0; i < path.length; i++){
    path[i].draw();
  }
  if(showShell){
    for(int i = 0; i < path.length; i++){
      if (path[i].active){
      fill(cols[i]);
        for(int j = 0; j < cylinders[i].length; j++){
          cylinders[i][j].draw();
        }
      }
    }
  }
  popMatrix();
  noFill();
  stroke(150);
  rect(0,0,wide,high);
  ellipse(arcball.center_x, arcball.center_y, arcball.radius, arcball.radius);
  for(int i = 0; i < button.length; i++){
    button[i].draw();
  }
  if(sim){
    simu();
  }
}

void unpack(String value) {
  //println(value);
  int [] unpacking = int(split(value,":"));
  for(int i = 0; i < unpacking.length; i+=3){
    int step = i/3;
    if(path[step].active){
      path[step].track(unpacking[i],unpacking[i+1],unpacking[i+2]);
      if(record){
        path[step].addVector(unpacking[i],unpacking[i+1],unpacking[i+2]);
      }
    }
  }
}
void simu(){
  String [] seed = {
    ""+(path[0].x[path[0].x.length-1]+int(random(-4,4)*10)),""+(path[0].y[path[0].y.length-1]+int(random(-4,4)*10)),""+(path[0].z[path[0].z.length-1]+int(random(-4,4)*10)),
    ""+int(random(640)),""+int(random(480)),""+int(random(480)),
    ""+int(random(640)),""+int(random(480)),""+int(random(480)),
    ""+int(random(640)),""+int(random(480)),""+int(random(480)),
    ""+int(random(640)),""+int(random(480)),""+int(random(480)),
    ""+int(random(640)),""+int(random(480)),""+int(random(480))
  };
  String pass = join(seed,":");
  unpack(pass);
}
